diff -r 7d0ce7586c36 src/lib-storage/index/dbox-multi/mdbox-mail.c
--- a/src/lib-storage/index/dbox-multi/mdbox-mail.c	Tue Jun 05 00:34:32 2012 +0300
+++ b/src/lib-storage/index/dbox-multi/mdbox-mail.c	Thu Jun 07 22:32:30 2012 +0300
@@ -170,6 +170,32 @@
 	return 0;
 }
 
+static int
+mdbox_mail_get_special(struct mail *_mail, enum mail_fetch_field field,
+		       const char **value_r)
+{
+	struct dbox_mail *mail = (struct dbox_mail *)_mail;
+	struct mdbox_mailbox *mbox =
+		(struct mdbox_mailbox *)_mail->transaction->box;
+	struct mdbox_map_mail_index_record rec;
+	uint32_t map_uid;
+	uint16_t refcount;
+
+	switch (field) {
+	case MAIL_FETCH_REFCOUNT:
+		if (mdbox_mail_lookup(mbox, _mail->transaction->view,
+				      _mail->seq, &map_uid) < 0)
+			return -1;
+		if (mdbox_map_lookup_full(mbox->storage->map, map_uid,
+					  &rec, &refcount) < 0)
+			return -1;
+		*value_r = p_strdup_printf(mail->imail.data_pool, "%u", refcount);
+		return 0;
+	default:
+		return dbox_mail_get_special(_mail, field, value_r);
+	}
+}
+
 static void
 mdbox_mail_update_flags(struct mail *mail, enum modify_type modify_type,
 			enum mail_flags flags)
@@ -208,7 +234,7 @@
 	index_mail_get_headers,
 	index_mail_get_header_stream,
 	dbox_mail_get_stream,
-	dbox_mail_get_special,
+	mdbox_mail_get_special,
 	index_mail_get_real_mail,
 	mdbox_mail_update_flags,
 	index_mail_update_keywords,
diff -r 7d0ce7586c36 src/lib-storage/index/maildir/maildir-mail.c
--- a/src/lib-storage/index/maildir/maildir-mail.c	Tue Jun 05 00:34:32 2012 +0300
+++ b/src/lib-storage/index/maildir/maildir-mail.c	Thu Jun 07 22:32:30 2012 +0300
@@ -476,6 +476,7 @@
 	struct index_mail *mail = (struct index_mail *)_mail;
 	struct maildir_mailbox *mbox = (struct maildir_mailbox *)_mail->box;
 	const char *path, *fname = NULL, *end, *guid, *uidl, *order;
+	struct stat st;
 
 	switch (field) {
 	case MAIL_FETCH_GUID:
@@ -560,6 +561,12 @@
 			*value_r = p_strdup(mail->data_pool, order);
 		}
 		return 0;
+	case MAIL_FETCH_REFCOUNT:
+		if (maildir_mail_stat(_mail, &st) < 0)
+			return -1;
+		*value_r = p_strdup_printf(mail->data_pool, "%lu",
+					   (unsigned long)st.st_nlink);
+		return 0;
 	default:
 		return index_mail_get_special(_mail, field, value_r);
 	}
diff -r 7d0ce7586c36 src/lib-storage/mail-storage.h
--- a/src/lib-storage/mail-storage.h	Tue Jun 05 00:34:32 2012 +0300
+++ b/src/lib-storage/mail-storage.h	Thu Jun 07 22:32:30 2012 +0300
@@ -141,7 +141,8 @@
 	MAIL_FETCH_MAILBOX_NAME		= 0x00080000,
 	MAIL_FETCH_SEARCH_RELEVANCY	= 0x00100000,
 	MAIL_FETCH_GUID			= 0x00200000,
-	MAIL_FETCH_POP3_ORDER		= 0x00400000
+	MAIL_FETCH_POP3_ORDER		= 0x00400000,
+	MAIL_FETCH_REFCOUNT		= 0x00800000
 };
 
 enum mailbox_transaction_flags {
diff -r 7d0ce7586c36 src/plugins/lazy-expunge/lazy-expunge-plugin.c
--- a/src/plugins/lazy-expunge/lazy-expunge-plugin.c	Tue Jun 05 00:34:32 2012 +0300
+++ b/src/plugins/lazy-expunge/lazy-expunge-plugin.c	Thu Jun 07 22:32:30 2012 +0300
@@ -122,7 +122,22 @@
 	struct lazy_expunge_transaction *lt =
 		LAZY_EXPUNGE_CONTEXT(_mail->transaction);
 	struct mail_save_context *save_ctx;
-	const char *error;
+	const char *value, *error;
+	unsigned long refcount;
+
+	if (mail_get_special(_mail, MAIL_FETCH_REFCOUNT, &value) < 0) {
+		lt->failed = TRUE;
+		return;
+	}
+	if (*value == '\0')
+		refcount = 0;
+	else if (str_to_ulong(value, &refcount) < 0)
+		i_panic("Invalid mail refcount number: %s", value);
+	if (refcount > 1) {
+		/* this isn't the last instance of the mail */
+		mmail->super.expunge(_mail);
+		return;
+	}
 
 	if (lt->dest_box == NULL) {
 		lt->dest_box = mailbox_open_or_create(luser->lazy_ns->list,
